package com.example.what_the_fantasy.ui.screens

import android.os.Bundle
import android.text.style.BackgroundColorSpan
import android.util.Log
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Brush
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.PasswordVisualTransformation
import androidx.compose.ui.text.input.VisualTransformation
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.example.what_the_fantasy.ui.theme.What_The_FantasyTheme
import androidx.navigation.NavController
import androidx.navigation.Navigation
import androidx.navigation.compose.rememberNavController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import com.example.what_the_fantasy.R
import com.example.what_the_fantasy.data.local.UserStub
import com.example.what_the_fantasy.data.model.User
import com.example.what_the_fantasy.ui.components.SpaceHeightComponent
import com.example.what_the_fantasy.ui.components.TitlePageComponent
import org.mindrot.jbcrypt.BCrypt
import java.security.MessageDigest

@Composable
fun LoginPage(navController : NavController) {

    val users = UserStub.allUsers;
    val gradient = Brush.linearGradient(
        colors = listOf(Color(0xFF7B1FA2), Color(0xFF311B92)), // Violet clair → Violet foncé
        start = Offset(0f, 1000f), // Départ en bas à gauche
        end = Offset(1000f, 0f) // Fin en haut à droite
    )

    Box(
        modifier = Modifier
            .fillMaxSize()
            .background(Color(0xFF100C1B)),
        contentAlignment = Alignment.Center
    ){
        Column(
            modifier = Modifier
                .fillMaxWidth(0.9f) // Ajuste la largeur
                .padding(20.dp) // Marge extérieure
                .clip(RoundedCornerShape(16.dp)) // Arrondi les angles
                .background(gradient) // Ajoute un fond blanc
                .padding(20.dp), // Padding interne
            horizontalAlignment = Alignment.CenterHorizontally
        ) {

            TitlePageComponent(R.string.titleLogin, 20,Color.White)
            val identifiant =IdentifiantTextField(R.string.IdentifiantLogin)
            val passwd = PassWdTextField(R.string.PasswdLogin)
            SpaceHeightComponent(16)
            ConnexionButtonLogin(users,identifiant, passwd, R.string.ButtonLogin,18, Color.White, Color.Black,navController)
            SpaceHeightComponent(16)
            CreateAccountButton(R.string.ButtonCreateLogin,12, Color.White, navController)
        }
    }
}




@Composable
fun IdentifiantTextField(textIdentifiantResId : Int) : String{
    val textIdentifiant = stringResource(id = textIdentifiantResId)
    var identifiant by remember { mutableStateOf("") } // Stocke la valeur du champ
    Column(modifier = Modifier.padding(top = 16.dp)) {
        OutlinedTextField(
            value = identifiant,
            onValueChange = { identifiant = it },
            label = { Text(textIdentifiant) },
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = 8.dp),
            keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Text),
            shape = RoundedCornerShape(16.dp) // 🔹 Bords arrondis
        )
    }
    return identifiant;
}

@Composable
fun PassWdTextField(textpasswdResId : Int) : String{
    val textpasswd = stringResource(id = textpasswdResId)
    var passwd by remember { mutableStateOf("") } // Stocke la valeur du champ
    var passwordVisible by remember { mutableStateOf(false) } // État pour afficher/masquer
    Column(modifier = Modifier.padding(top = 10.dp)) {
        OutlinedTextField(
            value = passwd,
            onValueChange = { passwd = it },
            label = { Text(textpasswd) },
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = 8.dp),
            keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Password),
            visualTransformation = if (passwordVisible) VisualTransformation.None else PasswordVisualTransformation(),
            trailingIcon = {
                IconButton(onClick = { passwordVisible = !passwordVisible }) {
                }
            },
            shape = RoundedCornerShape(16.dp) // Bords arrondis
        )
    }
    return passwd;
}

//@Composable
//fun TitlePage(titleResId : Int, size : Int, color : Color){
//    val title = stringResource(id = titleResId)
//    Text(
//        text = title,
//        fontSize = size.sp,
//        fontWeight = FontWeight.Bold,
//        color = color
//    )
//}

@Composable
fun ConnexionButtonLogin(userStub : List<User>, id : String, passwd : String, titleResId : Int, size : Int, colorButton : Color, colorText : Color, navController: NavController){
    val title = stringResource(id = titleResId)
    Button(
        onClick = { validLogin(id, passwd, userStub, navController) },
        colors = ButtonDefaults.buttonColors(containerColor = colorButton),
        modifier = Modifier
            .fillMaxWidth(),
    ) {
        Text(title, fontSize = size.sp, color = colorText)
    }
}


fun validLogin(identifiant : String, passwd : String, users : List<User>, navController: NavController){
    users.forEach { user ->
        val hashPassWd = hashPassword(passwd)

        if (user.username == identifiant && user.password == hashPassWd){
            navController.navigate("profile")
            //navController.navigate(ProfilPage(navController))
        }
    }
}

fun hashPassword(password: String): String {
    // SHA-256
    val digest = MessageDigest.getInstance("SHA-256")

    // Convertir mdp en bytes et appliquer le hash
    val hashedBytes = digest.digest(password.toByteArray())

    // Convertir le tableau de bytes en une chaîne hexadécimale
    return hashedBytes.joinToString("") { "%02x".format(it) }
}


@Composable
fun CreateAccountButton(titleResId : Int, size : Int, color : Color, navController: NavController){
    val title = stringResource(id = titleResId)
    Text(
        text = title,
        fontSize = size.sp,
        color = color,
        modifier = Modifier.clickable {
            navController.navigate("signup")// rediriger vers la page de création de compte
        }
    )
}

@Composable
fun AppNavigator() {
    val navController = rememberNavController()

    NavHost(navController, startDestination = "login") {
        composable("login") { LoginPage(navController) }
        composable("signup") { SignUpPage(navController) }
        composable("profile") { ProfilPage(navController) }
    }

}